// -------------------------------------
// getCanvasData
// -------------------------------------

function getCanvasData()
{
   var canvas  = document.getElementById("the-canvas");
   var context = canvas.getContext("2d");

   var data =
   {
      "canvas" : canvas,
      "context": context,
      "width"  : canvas.width,
      "xCenter": (canvas.width / 2.0),
      "height" : canvas.height,
      "yCenter": (canvas.height / 2.0)
   };

   return(data);
}

// -------------------------------------
// G - Our global data structure, keeps
//     track of the current state of all
//     items we need to draw
// -------------------------------------

var G =
{
   circles    : [],
   mouseDown  : false,
   lastAddTime: null
};

// -------------------------------------
// draw - render the items listed in G
// -------------------------------------

function draw()
{
   // Draw every circle whose radius is still "large enough"

   var data = getCanvasData();

   for(var i = 0; i < G.circles.length; i ++)
   {
      if(G.circles[i].radius > 15)
      {
         // Move the circle a bit, shrink it
         // a bit and then draw

         var xNegative = (Math.random() < 0.5);
         var yNegative = (Math.random() < 0.5);

         var xDelta = Math.floor(Math.random() * 5);
         var yDelta = Math.floor(Math.random() * 5);

         if(xNegative)
            xDelta = -xDelta;

         if(yNegative)
            yDelta = -yDelta;

         G.circles[i].x += xDelta;
         G.circles[i].y += yDelta;

         G.circles[i].radius --;

         // Choose a random, pastel color for this circle

         var r = 127 + Math.floor(Math.random() * 128);
         var g = 127 + Math.floor(Math.random() * 128);
         var b = 127 + Math.floor(Math.random() * 128);

         data.context.fillStyle =
         "rgba(" + r + "," + g + "," + b + "," + "0.3" + ")";

         data.context.beginPath();

         data.context.arc(
            G.circles[i].x, G.circles[i].y,
            G.circles[i].radius, 0, 2 * Math.PI, true
         );

         data.context.fill();
         data.context.stroke();
      }
   }

   // Arrange to have 'draw' called again afer
   // a brief timeout

   window.setTimeout(
      function() { requestAnimationFrame(draw); },
      50
   );
}

// -------------------------------------
// mouseUp
// -------------------------------------

function mouseUp(event)
{
   G.mouseDown = false;
}

// -------------------------------------
// mouseDown
// -------------------------------------

function mouseDown(event)
{
   G.mouseDown = true;
}

// -------------------------------------
// mouseMove
// -------------------------------------

function mouseMove(event)
{
   if(!G.mouseDown)
      return;

   if(G.lastAddTime == null)
   {
      G.lastAddTime = new Date();
   }
   else
   {
      var now   = new Date();
      var delta = now - G.lastAddTime;

      if(delta < 75)
         return;

      G.lastAddTime = now;
   }

   var x = event.clientX;
   var y = event.clientY;

   var data = getCanvasData();
   var rect = data.canvas.getBoundingClientRect();

   x -= rect.left;
   y -= rect.top;

   // Now x and y are relative to our canvas

   G.circles.push(
      {
         x: x, y: y,
         radius: 25 + Math.floor(Math.random() * 25)
      }
   );
}

function mouseEnter(event)
{
   G.mouseDown = false;
}
